unit Entities;

interface

uses
  Aurelius.Mapping.Attributes, Aurelius.Types.Proxy, Aurelius.Types.Nullable,
  System.Generics.Collections;

type

  TMaster = class;
  TDetail = class;

  [ Entity, Automapping ]
  TMaster = class
  private
    FId   : integer;
    FName : string;
    FValue: string;
    [ ManyValuedAssociation( [ TAssociationProp.Lazy ], CascadeTypeAllRemoveOrphan, 'FParent' ) ]
    FItems: Proxy< TList< TDetail > >;
    function GetItems: TList< TDetail >;
  public
    constructor Create;
    destructor Destroy; override;
    property Id: integer read FId write FId;
    property Name: string read FName write FName;
    property Value: string read FValue write FValue;
    property Items: TList< TDetail > read GetItems;
  end;

  [ Entity, Automapping ]
  TDetail = class
  private
    FId   : integer;
    FName : string;
    FValue: string;
    [ Association( [ TAssociationProp.Lazy ], CascadeTypeAll ) ]
    FParent: Proxy< TMaster >;
    function GetParent: TMaster;
    procedure SetParent( const Value: TMaster );
  public
    property Id    : integer read FId write FId;
    property Name  : string read FName write FName;
    property Value : string read FValue write FValue;
    property Parent: TMaster read GetParent write SetParent;
  end;

implementation

{ TMaster }
constructor TMaster.Create;
begin
  inherited;
  FItems.SetInitialValue( TList< TDetail >.Create );
end;

destructor TMaster.Destroy;
begin
  inherited;
  FItems.DestroyValue;
end;

function TMaster.GetItems: TList< TDetail >;
begin
  Result := FItems.Value;
end;

{ TDetail }

function TDetail.GetParent: TMaster;
begin
  Result := FParent.Value;
end;

procedure TDetail.SetParent( const Value: TMaster );
begin
  FParent.Value := Value;
end;

initialization

RegisterEntity( TMaster );
RegisterEntity( TDetail );

end.
