unit EntityTests;

interface

uses
  TMSLoggingCore,
  Aurelius.Criteria.Linq, Aurelius.Mapping.Explorer, Aurelius.Events.Manager, Aurelius.Sql.SQLite, Aurelius.Drivers.Interfaces, Aurelius.Engine.DatabaseManager, Aurelius.Engine.ObjectManager,
  Aurelius.Drivers.SQLite, Aurelius.Comp.Drivers,
  Entities, DUnitX.TestFramework;

type

  [ TestFixture ]
  TEntityTests = class
  private
    Connection: IDBConnection;
    Manager   : TObjectManager;
  public
    [ SetupFixture ]
    procedure SetupFixture;
    [ TearDownFixture ]
    procedure TearDownFixture;
    [ Setup ]
    procedure Setup;
    [ TearDown ]
    procedure TearDown;
    [ Test ]
    procedure CascadeDeleteTest;
  end;

const
  cConnectionStr = 'Database=:memory:;EnableForeignKeys=True';

implementation

procedure TEntityTests.SetupFixture;
begin
  Connection := TSQLiteConnection.Create( cConnectionStr );
//  TMappingExplorer.Default.Events.OnSQLExecuting.Subscribe( procedure( Args: TSQLExecutingArgs )
//    begin
//      var
//      sLog := #13#10 + Args.Sql;
//      if Assigned( Args.Params ) then
//        for var p in Args.Params do
//          sLog := sLog + #13#10 + p.ToString;
//      sLog := sLog;
//      TMSLog( sLog );
//    end
//  );
  TDatabaseManager.Update( Connection );
end;

procedure TEntityTests.TearDownFixture;
begin

end;

procedure TEntityTests.Setup;
begin
  Manager := TObjectManager.Create( Connection );
end;

procedure TEntityTests.TearDown;
begin
  Manager.Free;
  Manager := nil;
end;

procedure TEntityTests.CascadeDeleteTest;
begin

                                                 var
  Master       := TMaster.Create;
  Master.Name  := 'Shopping List';
  Master.Value := 'Tescos';

  var
  Item       := TDetail.Create;
  Item.Name  := 'Tatties';
  Item.Value := '5 lbs';
  Master.Items.Add( Item );

  Item       := TDetail.Create;
  Item.Name  := 'Mince';
  Item.Value := '250 kilos';
  Master.Items.Add( Item );

  Item       := TDetail.Create;
  Item.Name  := 'Baked Beans';
  Item.Value := '6 pack';
  Master.Items.Add( Item );

  Manager.Save( Master );

  var
  ShoppingList := Manager.Find< TMaster >.List;
  try
    Assert.AreEqual( 1, ShoppingList.Count, 'One shopping list' );
    Assert.AreEqual( 3, ShoppingList.First.Items.Count, '3 items on the list' );
  finally
    ShoppingList.Free;
    ShoppingList := nil;
  end;

  Manager.Flush;
  Manager.Remove( Master );

  ShoppingList := Manager.Find< TMaster >.List;
  try
    Assert.AreEqual( 0, ShoppingList.Count, 'Shopping list deleted' );
  finally
    ShoppingList.Free;
  end;

end;

initialization

TDUnitX.RegisterTestFixture( TEntityTests );

end.
