unit ServerByHeader.Module;

interface

uses
  System.SysUtils, System.Classes, System.Generics.Collections,
  Aurelius.Drivers.SQLite,
  Aurelius.Drivers.Interfaces, XData.Server.Module, Sparkle.Comp.Server,
  Aurelius.Engine.DatabaseManager,
  Sparkle.Comp.JwtMiddleware, XData.Comp.Server,
  Sparkle.Comp.HttpSysDispatcher, Aurelius.Comp.Connection,
  Sparkle.Comp.CorsMiddleware, Sparkle.HttpServer.Module,
  Sparkle.HttpServer.Context,
  Aurelius.Tenancy.Interfaces,
  Aurelius.Tenancy.Pool, XData.Comp.ConnectionPool, Sparkle.Comp.TenantMiddleware;

type
  TServerByHeaderModule = class(TDataModule)
    SparkleHttpSysDispatcher1: TSparkleHttpSysDispatcher;
    XDataServer1: TXDataServer;
    XDataServer1CORS: TSparkleCorsMiddleware;
    MultiDBPool: TXDataMultiDBConnectionPool;
    XDataServer1Tenant: TSparkleTenantMiddleware;
    procedure DataModuleCreate(Sender: TObject);
    procedure MultiDBPoolCreateDatabasePool(Sender: TObject; const DatabaseId: string;
      var Pool: IDBConnectionPool);
  public
    { Public declarations }
    procedure StartServer;
    procedure StopServer;
    function IsRunning: Boolean;
    procedure AddTenant(DBName: string);
  end;

const
  DefaultServerUrl = 'http://localhost:2001/tms/multitenant/';

var
  ServerByHeaderModule: TServerByHeaderModule;

implementation

uses
  XData.Aurelius.ConnectionPool;


{%CLASSGROUP 'System.Classes.TPersistent'}

{$R *.dfm}

{ TServerByHeaderModule }

procedure TServerByHeaderModule.AddTenant(DBName: string);
var
  Pool: IDBConnectionPool;
  Conn: IDBConnection;
begin
  Pool := (MultiDBPool.GetPoolInterface as ITenantDBPoolProvider).GetPool(DBName);
  Conn := Pool.GetConnection;
  TDatabaseManager.Update(Conn);
end;

procedure TServerByHeaderModule.DataModuleCreate(Sender: TObject);
begin
  AddTenant('test');
end;

function TServerByHeaderModule.IsRunning: Boolean;
begin
  Result := SparkleHttpSysDispatcher1.Active;
end;

procedure TServerByHeaderModule.StartServer;
begin
  XDataServer1.BaseUrl := DefaultServerUrl;
  SparkleHttpSysDispatcher1.Start;
end;

procedure TServerByHeaderModule.StopServer;
begin
  SparkleHttpSysDispatcher1.Stop;
end;

procedure TServerByHeaderModule.MultiDBPoolCreateDatabasePool(Sender: TObject; const DatabaseId: string;
  var Pool: IDBConnectionPool);
begin
  // Just create your connection pool the way you do normally
  // Each connection pool should have its specific connection settings to point
  // to the correct database
  Pool := TDBConnectionPool.Create(10,
    function: IDBConnection
    begin
      Result := TSQLiteNativeConnectionAdapter.Create(DatabaseId + '.db');
    end
  );
end;

end.
