unit XDataTests;

interface

uses
  TMSLoggingCore,
  Aurelius.Events.Manager, Aurelius.Mapping.Explorer,
  XData.Client,
  Sparkle.Http.Client,
  System.IOUtils,
  DUnitX.TestFramework, ServerByHeader.Module, Entities;

type

  [ TestFixture ]
  TXDataTests = class
  private
    Server: TServerByHeaderModule;
    Client: TXDataClient;
  public

    [ SetupFixture ]
    procedure SetupFixture;
    [ TearDownFixture ]
    procedure TearDownFixture;

    [ Setup ]
    procedure Setup;
    [ TearDown ]
    procedure TearDown;

    [ Test ]
    procedure CascadeDeleteTest;
  end;

implementation

procedure TXDataTests.SetupFixture;
begin
  TMappingExplorer.Default.Events.OnSQLExecuting.Subscribe( procedure( Args: TSQLExecutingArgs )
    begin
      var
      sLog := #13#10 + Args.Sql;
      if Assigned( Args.Params ) then
        for var p in Args.Params do
          sLog := sLog + #13#10 + p.ToString;
      sLog := sLog;
      TMSLog( sLog );
    end
  );

  If TFile.Exists( 'Test.db' ) then
    TFile.Delete( 'Test.db' );
  Server := TServerByHeaderModule.Create( nil );
  Server.StartServer;
end;

procedure TXDataTests.TearDownFixture;
begin
  Server.StopServer;
  Server.Free;
end;

procedure TXDataTests.Setup;
begin
  Client                             := TXDataClient.Create;
  Client.HttpClient.OnSendingRequest := procedure( Req: THttpRequest )
    begin
      Req.Headers.SetValue( 'database-name', 'test' );
    end;
  Client.Uri := 'http://localhost:2001/tms/multitenant/';
end;

procedure TXDataTests.TearDown;
begin
  Client.Free;
end;

procedure TXDataTests.CascadeDeleteTest;
begin

  var
  Master       := TMaster.Create;
  Master.Name  := 'Shopping List';
  Master.Value := 'Tescos';

  var
  Item       := TDetail.Create;
  Item.Name  := 'Tatties';
  Item.Value := '5 lbs';
  Master.Items.Add( Item );

  Item       := TDetail.Create;
  Item.Name  := 'Mince';
  Item.Value := '250 kilos';
  Master.Items.Add( Item );

  Item       := TDetail.Create;
  Item.Name  := 'Baked Beans';
  Item.Value := '6 pack';
  Master.Items.Add( Item );

  Client.Post( Master );

  var
  ShoppingList := Client.List< TMaster >;
  try
    Assert.AreEqual( 1, ShoppingList.Count, 'One shopping list' );
    Assert.AreEqual( 3, ShoppingList.First.Items.Count, '3 items on the list' );
  finally
    ShoppingList.Free;
    ShoppingList := nil;
  end;

  Client.Delete( Master );

  ShoppingList := Client.List< TMaster >;
  try
    Assert.AreEqual( 0, ShoppingList.Count, 'Shopping list deleted' );
  finally
    ShoppingList.Free;
  end;

end;

initialization

TDUnitX.RegisterTestFixture( TXDataTests );

end.
